﻿/**
* Main game class for episode 18 of One of those Weeks.
* Holds all the flags and data so different scenes all have access to all necessary information.
*
* @author Billy D. Spelchan
* @version 1.00.060620
*/
class OotW18
{
	// ***** CONSTANTS *****
	public static var MESSAGEHIDE:Number = 1;
	public static var TILESET:String = "-klmjihg";
	public static var SAND_TILE:Number = 8;
	
	// ***** VARIABLES *****
	
	// Lets the game scene know if it should start in the cockpit with a message or in the orchard
	// The orchard feature is due to the fact that the game starts in the cockpit but other scenes
	// go to the orchard (but going directly means that the first frame actionscript is not known)
	public var startingFlag:Boolean = true;
	public var haveOrange:Boolean = false;
	public var haveGPS:Boolean = false;
	public var havePen:Boolean = false;
	public var haveNotebook:Boolean = false;
	public var seenLand:Boolean = false;
	public var seenCave:Boolean = false;
	
	public var RAW_MAP:Array;
	public var dirString:Array;
	public var tdTilemap:Array;
	public var automapData:Array;
	public var playerX:Number = 1;
	public var playerY:Number = 30;
	public var playerD:Number = 2;
	
	// ***** CONSTRUCTOR *****
	public function OotW18()
	{
		RAW_MAP = new Array(
//1234567890123456789012345678901
"jjjjjjjjjjjjjjjjjjjjjjjjjjjjjjjj",
"klmmkkmmllmkmllkmklmkmlmklmlk--k",
"lkklk-----mlm---------k-l---lh-m",
"kmkmk-k-l--m--mkmlkk--l---k---lk",
"lkmkl-mklk---km----ll-km--mlmkkm",
"kkmlk---kmlk--l-k---m--lk----mlk",
"llmgm-klm--k-lk-m-lk----mk-m-kkm",
"mkl-m--m---l-km-lk---lk--mkl-llk",
"km--k-lm-k-m--l--m-klm--lkmm-kkm",
"l--k--m----k--mk---m----k-----lk",
"k--m--lkm--mk--lm-lm--klm-kl-kmm",
"lm-k-k--l-kml---k-m--km--k-m-lkk",
"mk-m-m--k--mkkm-lmk--m--lm---kml",
"l--k---ml-klm----k--ll---k--klkk",
"l-mm--lk--l---kl-m--m-km-k--lkml",
"k--l-km---k--lm--mk-l--m-lm-mlkm",
"mk---m--lkmk---k--m-kl---mk-kmll",
"mlk--k-mm--lk--m--k--ml---m-mkkm",
"kkm----k-m--kk-m--km--m-k-m--lkm",
"mkll-m-k----m---k---m-k--ml--klk",
"kmkm---mk-l-kll-mmkml-l--k---lkl",
"mkkm-l-mm--mk-l----k--m--m-k-mmm",
"kll---mk-m-l--ml-k-m--mk-l----kl",
"lmk---k----ml--k-m-mk--m-mkk-mmk",
"kkm-m-k-k-km--km-m--l--k---l--kl",
"lkk---m-l-mk-kl--lk-mkkml--k--lk",
"m---l-mm--l---m---ml----m-mll-km",
"lk--mmkll-mkm-l-k--m-lm----mk--l",
"lm-lmlmmm-ll--l-l--l-m---m-l---m",
"l------m-----mkkmm---k--lk---kml",
"m---lk---mllkmlmlm-----mkkm-ll-k",
"mimlmmlkkmlmkkllkkmlmllmkmlmm--m"
//1234567890123456789012345678901
						);
		dirString = new Array( "North", "East", "South", "West");
		tdTilemap = new Array(32);
		automapData = new Array;
		for (var cntr:Number = 0; cntr < 32; ++cntr)
			automapData[cntr] = new Array(32);
	}
	
	public function restartGame()
	{
		var cntr:Number;
		
		startingFlag = true;		
		haveOrange = false;
		haveGPS = false;
		havePen = false;
		haveNotebook = false;
		seenLand = false;
		seenCave = false;

		setOrchardCoordinates();
		for (cntr = 0; cntr < 32; ++cntr)
			tdTilemap[cntr] = RAW_MAP[cntr];
		for (var cntrY:Number = 0; cntrY < 32; ++cntrY)
			for (var cntrX:Number = 0; cntrX < 32; ++cntrX)
				automapData[cntrY][cntrX] = 0;
	}

	function setOrchardCoordinates()
	{
		playerX = 29;
		playerY = 3;
		playerD = 2;
	}
	
	function setBeachCoordinates()
	{
		playerX = 1;
		playerY = 30;
		playerD = 0;
	}
	
	function getOffsetTile(dist:Number, lr:Number):Number
	{
		var tx:Number = playerX;
		var ty:Number = playerY;
		var lradj:Number = lr;
		switch (playerD)
		{
			case 0: // north
				tx += lradj;
				ty -= dist;
				break;
			case 1: // east
				tx += dist;
				ty += lradj;
				break;
			case 2: // south
				tx -= lradj;
				ty += dist;
				break;
			case 3: // west
				tx -= dist;
				ty -= lradj;
				break;
		}
		while(tx < 0) tx += 32;
		while(tx > 31) tx -= 32;
		while (ty < 0) ty += 32;
		if (ty > 31)
			return SAND_TILE;
	//	while (ty > 31) ty -= 32;
		var tchar:String = tdTilemap[ty].charAt(tx);
		var tval:Number = TILESET.indexOf(tchar);
		return tval;
	}

	function canMoveForward()
	{
		var canEnter:Boolean = true;
		var tileType:Number = getOffsetTile(1, 0);
		if ((tileType >= 1) && (tileType <= 3)) // tree
			canEnter = false;
//		else if (tileType == 5) // Charles Chink
//			canEnter = false;
		else if (tileType == 7) // Cave
		{
			canEnter = false;
		}
		return canEnter;
	}
	
	function enterTile(nx:Number, ny:Number):Number
	{		
		var id:Number = getOffsetTile(0,0);
		var tempTile:Number;
		trace ("Tile value is " + id);
		var tx:Number, ty:Number;
		var tchar:String;
		for (var cntrX:Number = -3; cntrX < 4; ++cntrX) {
			tx = nx+cntrX;
			if (tx < 0) continue;
			if (tx > 31) break;
			for (var cntrY:Number = -3; cntrY < 4; ++cntrY) {
				ty = (ny+cntrY);
				if (ty < 0) continue;
				if (ty > 31) break;
				tchar = tdTilemap[ty].charAt(tx);
				tempTile = TILESET.indexOf(tchar);
				automapData[ny+cntrY][nx+cntrX] = tempTile + 1; // remember 0 is used for unseen tile
			}
		}
		return id;
	}
	
	function getMaxScore():Number
	{
		return 20;
	}
	
	function getFinalScore():Number
	{
		var tilesExplored:Number = 0;
		var score:Number = 0;
		for (var cntrX:Number = 0; cntrX < 32; ++cntrX) {
			for (var cntrY:Number = 0; cntrY < 32; ++cntrY) {
				if (automapData[cntrY][cntrX] > 0)
					++tilesExplored;
			}
		}
		trace ("Tiles explored: " + tilesExplored);
		if (tilesExplored > 900)
			score = 10;
		else
			score = Math.floor(tilesExplored / 100);
		if (haveOrange) // won
			score += 5;
		if (seenLand) // bonus earned by standing on rock
			score += 3;
		if (seenCave) // bonus for noticing cave link
			score += 2; 
		return score;
	}
}